% Versuch 1_4
% C. Roppel, HS Schmalkalden
% MATLAB- und Scilab-bungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2023
% Getestet mit MATLAB R2020a
%%
clear
clc
close all

%% Parameter
alpha = 0.5;  % Roll-off-Faktor
os = 8;       % Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;   % Verzgerung des Wurzel-Kosinus-roll-off-Filters
nSym = 1000;  % Anzahl Symbole
EbN0 = 6;     % Eb/N0 in dB

%% Wurzel-Kosinus-roll-off-Filter
psrc = rcosdesign(alpha, 2*dly_rc, os, 'sqrt');

%% Basisbandsignal
rng(1234);  % Initialisiere Zufallsgenerator
bits = randi([0 1], nSym, 1);  % Spaltenvektor der Lnge nSym mit Zufallszahlen 0 oder 1
symbols = 2*bits - 1;  % Binre bipolare Symbolfolge
symbols = upsample(symbols, os);  % Einfgen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, symbols);  % Faltung (convolution)
x = x(dly_rc*os + 1:end - dly_rc*os);  % Entfernen der ersten und letzten Werte

%% AWGN-Kanal
snr = 10*log10(2) + EbN0 - 10*log10(os);  % Strabstand in dB
sigPwr = mean(x.^2);  % Signalleistung
noisePwr = sigPwr/(10^(snr/10));  % Rauschleistung
rng(1234); % Initialisiere Zufallsgenerator
noise = randn(length(x), 1)*sqrt(noisePwr);
y = x + noise;  % Signal + Rauschen

eyediagram(y, 2*os, 2);

figure; 
periodogram(y, [], [], os, 'centered');

%% Empfnger mit signalangepasstem Filter
ye = conv(psrc, y);
ye = ye(dly_rc*os + 1:end - dly_rc*os);

eyediagram(ye, 2*os, 2);

ysym = downsample(ye, os);  % Herabsetzen der Abtastrate auf 1 Sample pro Symbol
ysym = sign(ysym);  % Entscheider
ybit = round((ysym + 1)/2);

% Bestimmung der Bitfehlerhufigkeit
[nerror, ber] = biterr(ybit, bits);
berTheo = 0.5*erfc(sqrt(10^(EbN0/10)));
fprintf('Eb/N0: %f dB  Pb: %f  Bitfehler: %d  Bitfehlerhufigkeit: %f\n', EbN0, berTheo, nerror, ber);
