% Versuch 1_5
% C. Roppel, HS Schmalkalden
% MATLAB- und Scilab-bungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2023
% Getestet mit MATLAB R2020a
%%
clear
clc
close all

%% Parameter
alpha = 0.5;  % Roll-off-Faktor
os = 8;       % Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;   % Verzgerung des Wurzel-Kosinus-roll-off-Filters
nSym = 2e4;   % Anzahl Symbole
EbN0List = [4:8];  % Eb/N0 in dB
berTheoList = zeros(1, length(EbN0List));
berList = zeros(1, length(EbN0List));

%% Wurzel-Kosinus-roll-off-Filter
psrc = rcosdesign(alpha, 2*dly_rc, os, 'sqrt');

%% Basisbandsignal
rng(1234); % Initialisiere Zufallsgenerator
bits = randi([0 1], nSym, 1); % Spaltenvektor der Lnge nSym mit Zufallszahlen 0 oder 1
symbols = 2*bits - 1; % Binre bipolare Symbolfolge
symbols = upsample(symbols, os); % Einfgen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, symbols);  % Faltung (convolution)
x = x(dly_rc*os + 1:end - dly_rc*os);  % Entfernen der ersten und letzten Werte

%% Schleife fr verschiedene Eb/N0
for i = 1:length(EbN0List)
    % AWGN-Kanal
    EbN0 = EbN0List(i);
    snr = 10*log10(2) + EbN0 - 10*log10(os);
    sigPwr = mean(x.^2);
    noisePwr = sigPwr/(10^(snr/10));
    rng(1234); % Initialisiere Zufallsgenerator
    noise = randn(length(x), 1)*sqrt(noisePwr);
    y = x + noise;
    
    % Empfnger mit signalangepasstem Filter
    ye = conv(psrc, y);
    ye = ye(dly_rc*os + 1:end - dly_rc*os);
    ysym = downsample(ye, os);
    ysym = sign(ysym);
    ybit = round((ysym + 1)/2);
    
    % Bestimmung der Bitfehlerhufigkeit
    [nerror, ber] = biterr(ybit, bits);
    berList(i) = ber;
    berTheo = 0.5*erfc(sqrt(10^(EbN0/10)));
    berTheoList(i) = berTheo;
    fprintf('Eb/N0: %5.2f dB  Pb: %f  Bitfehler: %4d  Bitfehlerhufigkeit: %f\n', EbN0, berTheo, nerror, ber);
end

%% Plot BER vs. Eb/N0
figure; 
semilogy(EbN0List, berList, '-.or'); 
grid; 
xlabel('E_b/N_0 in dB'), 
ylabel('Bitfehlerhufigkeit');
hold on; 
plot(EbN0List, berTheoList, '-x'); 
hold off;
legend('Simulation', 'Theoretisch', 'Location', 'SouthWest');
