% Versuch 3_3
% C. Roppel, HS Schmalkalden
% MATLAB- und Scilab-bungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2023
% Getestet mit MATLAB R2020a
%%
clear
clc
close all

%% Parameter
alpha = 0.5;    % Roll-off-Faktor
os = 8;         % Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;     % Verzgerung des Wurzel-Kosinus-roll-off-Filters
nSym = 2e4;     % Anzahl Symbole
m = 4;          % m-wertige Symbole
k = log2(m);    % k Bit pro Symbol
lambda = pi/4;  % QPSK Phasenoffset
EbN0List = [4:8];  % Eb/N0 in dB
serTheoList = zeros(1, length(EbN0List));
serList = zeros(1, length(EbN0List));

%% Wurzel-Kosinus-roll-off-Filter
psrc = rcosdesign(alpha, 2*dly_rc, os, 'sqrt');

%% QPSK-Modulator und Pulsformfilter
rng(1234);  % Initialisiere Zufallsgenerator
symbols = randi([0 (m - 1)], nSym, 1);  % Spaltenvektor der Lnge nSym mit Zufallszahlen im Bereich 0 bis m-1

xmod = pskmod(symbols, m, lambda);  % Modulation

xmod = upsample(xmod, os);  % Einfgen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, xmod);  % Faltung (convolution)
x = x(dly_rc*os + 1:end - dly_rc*os);  % Entfernen der ersten und letzten Werte

%% Mehrwegekanal
dt = 2;   % Laufzeitdifferenz (in Samples)
a = 0.8;  % Dmpfung des indirekten Pfades
b = 1;    % Produkt f_c * t_0

% Impulsantwort
hKanal = zeros(1, dt + 1);
hKanal(1) = 1;
hKanal(dt + 1) = a*exp(-j*2*pi*b);

% Faltung mit der Kanalimpulsantwort
r = conv(hKanal, x);

% Downsampling des Eingangssignals auf 2 Samples/Symbol
r = downsample(r, os/2);

%% T/2-Entzerrer
Ne = 33;                   % Anzahl der Koeffizienten
dly_eq = round((Ne-1)/2);  % Verzgerung des Entzerrers

% Zu entzerrender Impuls g(n)
gn = conv(psrc, hKanal);
gn = downsample(gn, os/2);

% Rauschleistung
var_r = 0.04;
% Faltungsmatrix mit m + 1 + N Zeilen und m + 1 Spalten
gn = gn(:);
F = toeplitz([gn; zeros(Ne-1, 1)], [gn(1) zeros(1, Ne-1)]);
% F hat z Zeilen und s Spalten, Weglassen jeder zweiten Zeile
[z, s] = size(F);
F(2:2:z, :) = [];
% Ideales Ausgangssignal
[z, s] = size(F);
yid = zeros(z, 1);
yid(round(z/2)) = 1;
% Berechnung der Koeffizienten
I = diag(ones(1, s));
en = (F'*F + var_r*I)\F'*yid;

%% Schleife fr verschiedene Eb/N0
for i = 1:length(EbN0List)
    % AWGN-Kanal
    EbN0 = EbN0List(i);
    snr = EbN0 + 10*log10(k) - 10*log10(2);  % Strabstand in dB
    sigPwr = 0.5*mean(abs(r).^2);  % Signalleistung
    noisePwr = sigPwr/(10^(snr/10));  % Rauschleistung
    rng(1234);
    noise_i = randn(length(r), 1)*sqrt(noisePwr);
    rng(4321);
    noise_q = randn(length(r), 1)*sqrt(noisePwr);
    rn = r + noise_i + j*noise_q;  % Signal + Rauschen
    
    % Entzerrer und QPSK-Demodulator
    y = conv(en, rn);
    y = y(dly_eq + 1:end - dly_eq);

    y = downsample(y, 2);
     
    ysym = pskdemod(y, m, lambda);
    
    % Bestimmung der Symbolfehlerhufigkeit
    ysym = ysym(1:length(symbols));
    [nerror, ser] = symerr(symbols, ysym);
    serList(i) = ser;
    if m <= 4
        serTheo = erfc(sqrt(10^(EbN0/10)));
    else
        serTheo = erfc(sqrt(log2(m)*10^(EbN0/10))*sin(pi/m));
    end
    serTheoList(i) = serTheo;
    fprintf('Eb/N0: %5.2f dB  Ps: %f  Symbolfehler: %4d  Symbolfehlerhufigkeit: %f\n', EbN0, serTheo, nerror, ser);
end

%% Plot SER vs. Eb/N0
figure; 
semilogy(EbN0List, serList, '-.or'); 
grid; 
xlabel('E_b/N_0 in dB'), 
ylabel('Symbolfehlerhufigkeit');
hold on; 
plot(EbN0List, serTheoList, '-x'); 
hold off;
legend('Simulation', 'Theoretisch', 'Location', 'SouthWest');

